<?php

/**
 * JCH Optimize - Performs several front-end optimizations for fast downloads.
 *
 * @author    Samuel Marshall <samuel@jch-optimize.net>
 * @copyright Copyright (c) 2020 Samuel Marshall / JCH Optimize
 * @license   GNU/GPLv3, or later. See LICENSE file
 *
 * If LICENSE file missing, see <http://www.gnu.org/licenses/>.
 */

namespace JchOptimize\Html;

use JchOptimize\ContainerFactory;
use JchOptimize\Core\Admin\MultiSelectItems;

use function __;
use function array_key_exists;
use function array_unshift;
use function call_user_func;
use function call_user_func_array;
use function esc_attr_e;
use function explode;
use function get_option;
use function in_array;
use function is_multisite;
use function ucfirst;

class Helper
{
    /**
     * @param array|string $defaultValue
     * @param mixed        ...$args
     *
     * @psalm-suppress InvalidGlobal
     */
    public static function _(string $key, string $settingName, $defaultValue, ...$args): void
    {
        if (\defined('JCH_OPTIMIZE_GET_WORDPRESS_SETTINGS')) {
            global $jchParams;
            $jchParams[$settingName] = $defaultValue;

            return;
        }
        list($function, $proOnly) = static::extract($key);
        if ($proOnly && !\JCH_PRO) {
            $html = '<div>
                         <em style="padding: 5px; background-color: white; border: 1px #ccc;">'.__('Only available in Pro Version!', 'jch-optimize').'  </em>
                    </div>';
            echo $html;

            return;
        }
        $aSavedSettings = get_option('jch-optimize_settings');
        if (!isset($aSavedSettings[$settingName])) {
            $activeValue = $defaultValue;
        } else {
            $activeValue = $aSavedSettings[$settingName];
        }
        $callable = [__CLASS__, $function];
        // prepend $settingName, $activeValue to arguments
        array_unshift($args, $settingName, $activeValue);
        call_user_func_array($callable, $args);
    }

    public static function description(string $title, string $description, bool $new = \false): string
    {
        $html = '<div class="title">'.$title;
        if ($description) {
            $html .= '<div class="description">
                          <div><p>'.$description.'</p></div>
                      </div>';
        }
        if ($new) {
            $html .= '<span class="badge badge-danger">New!</span>';
        }
        $html .= '</div>';

        return $html;
    }

    public static function radio(string $settingName, string $activeValue, string $class = ''): void
    {
        $disabled = 'pro_capture_cache_enable' == $settingName && is_multisite() ? 'disabled' : '';
        ?>
        <fieldset id="jch-optimize_settings_<?php
        echo $settingName;
        ?>" class="btn-group <?php
        echo $class;
        ?>"
                  role="group" aria-label="Radio toggle button group">
            <input type="radio" id="jch-optimize_settings_<?php
        echo $settingName;
        ?>0"
                   name="<?php
        echo "jch-optimize_settings[{$settingName}]";
        ?>" class="btn-check" value="0"
                <?php
        echo '0' == $activeValue ? 'checked' : '';
        ?> <?php
        echo $disabled;
        ?> >
            <label for="jch-optimize_settings_<?php
        echo $settingName;
        ?>0"
                   class="btn btn-outline-secondary"><?php
        _e('No', 'jch-optimize');
        ?></label>
            <input type="radio" id="jch-optimize_settings_<?php
        echo $settingName;
        ?>1"
                   name="<?php
        echo "jch-optimize_settings[{$settingName}]";
        ?>" class="btn-check" value="1"
                <?php
        echo '1' == $activeValue ? 'checked' : '';
        ?> <?php
        echo $disabled;
        ?> >
            <label for="jch-optimize_settings_<?php
        echo $settingName;
        ?>1"
                   class="btn btn-outline-secondary"><?php
        _e('Yes', 'jch-optimize');
        ?></label>
        </fieldset>
        <?php
        if ('pro_smart_combine' == $settingName) {
            $imgUrl = JCH_PLUGIN_URL.'media/core/images/exclude-loader.gif';
            ?>
            <img id="img-<?php
            echo $settingName;
            ?>" src="<?php
            echo $imgUrl;
            ?>" style="display: none;">
            <button id="btn-<?php
            echo $settingName;
            ?>" type="button" class="btn btn-outline-secondary"
                    style="display: none;">Reprocess Smart Combine
            </button>
            <?php
        }
    }

    public static function select(string $settingName, string $activeValue, array $options, string $class = '', array $conditions = []): void
    {
        ?>
        <select id="jch-optimize_settings_<?php
        echo $settingName;
        ?>"
                name="<?php
        echo "jch-optimize_settings[{$settingName}]";
        ?>"
                class="chzn-custom-value <?php
        echo $class;
        ?>">
            <?php
        echo self::option($options, $activeValue, $conditions);
        ?>
        </select>
        <?php
    }

    /**
     * @param array<int|string, string> $options
     */
    public static function option(array $options, ?string $activeValue, array $conditions = []): string
    {
        $html = '';
        foreach ($options as $key => $value) {
            $selected = $activeValue == $key ? ' selected' : '';
            $disabled = '';
            if (!empty($conditions) && array_key_exists($key, $conditions)) {
                if (!call_user_func($conditions[$key])) {
                    $disabled = ' disabled';
                }
            }
            $html .= '<option value="'.esc_attr($key).'"'.$selected.$disabled.'>'.$value.'</option>';
        }

        return $html;
    }

    public static function multiselect(string $settingName, array $activeValues, string $type, string $group, string $class = ''): void
    {
        $container = ContainerFactory::getContainer();
        $multiSelect = $container->buildObject(MultiSelectItems::class);
        ?>
        <select id="jch-optimize_settings_<?php
        echo $settingName;
        ?>"
                name="<?php
        echo "jch-optimize_settings[{$settingName}]";
        ?>[]"
                class="jch-multiselect chzn-custom-value <?php
        echo $class;
        ?>" multiple="multiple" size="5"
                data-jch_type="<?php
        echo $type;
        ?>" data-jch_group="<?php
        echo $group;
        ?>"
                data-jch_param="<?php
        echo $settingName;
        ?>">

            <?php
        foreach ($activeValues as $value) {
            $option = $multiSelect->{'prepare'.ucfirst($group).'Values'}($value);
            ?>
                <option value="<?php
            esc_attr_e($value);
            ?>" selected><?php
            echo $option;
            ?></option>
                <?php
        }
        ?>
        </select>
        <img id="img-<?php
        echo $settingName;
        ?>" class="jch-multiselect-loading-image"
             src="<?php
        echo JCH_PLUGIN_URL.'media/core/images/exclude-loader.gif';
        ?>"/>
        <button id="btn-<?php
        echo $settingName;
        ?>" style="display: none;"
                class="btn btn-secondary btn-sm jch-multiselect-add-button" type="button"
                onmousedown="jchMultiselect.addJchOption('jch-optimize_settings_<?php
        echo $settingName;
        ?>')"><?php
        _e('Add item', 'jch-optimize');
        ?></button>
        <?php
    }

    public static function multiselectjs(string $settingName, array $activeValues, string $type, string $group, string $valueType, string $class = ''): void
    {
        $container = ContainerFactory::getContainer();
        $multiSelect = $container->buildObject(MultiSelectItems::class);
        $nextIndex = \count($activeValues);
        $i = 0;
        ?>
        <fieldset id="fieldset-<?php
        echo $settingName;
        ?>" data-index="<?php
        echo $nextIndex;
        ?>">
            <div class="jch-js-fieldset-children jch-js-excludes-header">
                <span class="jch-js-ieo-header">&nbsp;&nbsp;Ignore execution order&nbsp;&nbsp;&nbsp;</span>
                <span class="jch-js-dontmove-header">&nbsp;&nbsp;&nbsp;Don't move to bottom&nbsp;&nbsp;</span>
            </div>
            <?php
        foreach ($activeValues as $value) {
            if (!isset($value[$valueType]) || !\is_string($value[$valueType])) {
                continue;
            }
            $ieoChecked = isset($value['ieo']) ? 'checked' : '';
            $dontMoveChecked = isset($value['dontmove']) ? 'checked' : '';

            /** @var string $dataValue */
            $dataValue = $multiSelect->{'prepare'.ucfirst($group).'Values'}($value[$valueType]);
            ?>
                <div id="div-<?php
            echo $settingName;
            ?>-<?php
            echo $i;
            ?>"
                     class="jch-js-fieldset-children jch-js-excludes-container">
                        <span class="jch-js-excludes">
                            <span>
                                <input type="text" readonly value="<?php
            esc_attr_e($value[$valueType]);
            ?>"
                                       name="<?php
            echo "jch-optimize_settings[{$settingName}][{$i}][{$valueType}]";
            ?>">
                                       <?php
            echo $dataValue;
            ?>
                                <button type="button" class="jch-multiselect-remove-button"
                                        onmouseup="jchMultiselect.removeJchJsOption('div-<?php
            echo $settingName;
            ?>-<?php
            echo $i;
            ?>', 'jch-optimize_settings_<?php
            echo $settingName;
            ?>')"></button>
                            </span>
                        </span>
                    <span class="jch-js-ieo">
                            <input type="checkbox"
                                   name="<?php
            echo "jch-optimize_settings[{$settingName}][{$i}][ieo]";
            ?>" <?php
            echo $ieoChecked;
            ?>>
                        </span>
                    <span class="jch-js-dontmove">
                            <input type="checkbox"
                                   name="<?php
            echo "jch-optimize_settings[{$settingName}][{$i}][dontmove]";
            ?>" <?php
            echo $dontMoveChecked;
            ?>>
                        </span>
                </div>
                <?php
            ++$i;
        }
        ?>
        </fieldset>
        <select id="jch-optimize_settings_<?php
        echo $settingName;
        ?>"
                name="<?php
        echo "jch-optimize_settings[{$settingName}]";
        ?>[]"
                class="jch-multiselect chzn-custom-value <?php
        echo $class;
        ?>" multiple="multiple" size="5"
                data-jch_type="<?php
        echo $type;
        ?>" data-jch_group="<?php
        echo $group;
        ?>"
                data-jch_param="<?php
        echo $settingName;
        ?>">
        </select>
        <img id="img-<?php
        echo $settingName;
        ?>" class="jch-multiselect-loading-image"
             src="<?php
        echo JCH_PLUGIN_URL.'media/core/images/exclude-loader.gif';
        ?>"/>
        <button id="btn-<?php
        echo $settingName;
        ?>" style="display: none;"
                class="btn btn-secondary btn-sm jch-multiselect-add-button" type="button"
                onmousedown="jchMultiselect.addJchJsOption('jch-optimize_settings_<?php
        echo $settingName;
        ?>', '<?php
        echo $settingName;
        ?>', '<?php
        echo $valueType;
        ?>')"><?php
        _e('Add item', 'jch-optimize');
        ?></button>
        <script>
            jQuery('#jch-optimize_settings_<?php
        echo $settingName;
        ?>').on('change', function (evt, params) {
                jchMultiselect.appendJchJsOption('jch-optimize_settings_<?php
        echo $settingName;
        ?>', '<?php
        echo $settingName;
        ?>', params, '<?php
        echo $valueType;
        ?>')
            })
        </script>
        <?php
    }

    public static function text(string $settingName, string $activeValue, string $size = '30', string $class = ''): void
    {
        ?>
        <input type="text" id="jch-optimize_settings_<?php
        echo $settingName;
        ?>"
               name="<?php
        echo "jch-optimize_settings[{$settingName}]";
        ?>"
               value="<?php
        esc_attr_e($activeValue);
        ?>" size="<?php
        echo $size;
        ?>" class="<?php
        echo $class;
        ?>">
        <?php
    }

    public static function hidden(string $settingName, string $activeValue): void
    {
        ?>
        <input type="hidden" id="jch-optimize_settings_<?php
        echo $settingName;
        ?>"
               name="<?php
        echo "jch-optimize_settings[{$settingName}]";
        ?>"
               value="<?php
        esc_attr_e($activeValue);
        ?>">
        <?php
    }

    public static function input(string $settingName, string $activeValue, string $type = 'text', array $attr = []): void
    {
        ?>
        <input type="<?php
        echo $type;
        ?>"
               id="jch-optimize_settings_<?php
        echo $settingName;
        ?>"
               name="<?php
        echo "jch-optimize_settings[{$settingName}]";
        ?>"
               value="<?php
        esc_attr_e($activeValue);
        ?>"
            <?php
        echo self::attrArrayToString($attr);
        ?>>

        <?php
    }

    public static function checkbox(string $settingName, string $activeValue, string $class = ''): void
    {
        $checked = '1' == $activeValue ? 'checked="checked"' : '';
        ?>
        <input type="checkbox" id="jch-optimize_settings_<?php
        echo $settingName;
        ?>" class="<?php
        echo $class;
        ?>"
               name="<?php
        echo "jch-optimize_settings[{$settingName}]";
        ?>" data-toggle="toggle"
               data-onstyle="success" data-offstyle="danger" data-on="<?php
        _e('Yes', 'jch-optimize');
        ?>"
               data-off="<?php
        _e('No', 'jch-optimize');
        ?>" value="1" <?php
        echo $checked;
        ?>>
        <?php
    }

    /**
     * @param array<string, string> $options
     */
    public static function checkboxes(string $settingName, array $activeValues, array $options, string $class = ''): void
    {
        $i = '0';
        ?>
        <fieldset id="jch-optimize_settings_<?php
        echo $settingName;
        ?>" class="<?php
        echo $class;
        ?>">
            <ul>
                <?php
        foreach ($options as $key => $value) {
            $checked = in_array($key, $activeValues) ? 'checked' : '';
            ?>
                    <li>
                        <input type="checkbox" id="jch-optimize_settings_<?php
            echo $settingName.$i;
            ?>"
                               name="<?php
            echo "jch-optimize_settings[{$settingName}]";
            ?>[]"
                               value="<?php
            esc_attr_e($key);
            ?>" <?php
            echo $checked;
            ?>>
                        <label for="jch-optimize_settings_<?php
            echo $settingName.$i;
            ?>">
                            <?php
            echo $value;
            ?>
                        </label>

                    </li>
                    <?php
            ++$i;
        }
        ?>
            </ul>
        </fieldset>
        <?php
    }

    protected static function extract(string $key): array
    {
        $parts = explode('.', $key);
        $function = $parts[0];
        $proOnly = isset($parts[1]) && 'pro' === $parts[1];

        return [$function, $proOnly];
    }

    private static function attrArrayToString(array $attr): string
    {
        $attrString = '';
        foreach ($attr as $name => $value) {
            $attrString .= $name.'="'.$value.'" ';
        }

        return $attrString;
    }
}
